#include "BLKH.h"

static void MSTSP_transform();

/*
 * This file contains the main function of the BLKH_MSTSP program.
 */

int main(int argc, char *argv[])
{
    int Bottleneck, Bound, LowerBound;
    double Time, LastTime;

    // SetStackSize(63 * 1024 * 1024);
    /* Read the specification of the problem */
    if (argc >= 2)
        ParameterFileName = argv[1];
    MSTSP = 1;
    ReadParameters();
    ReadProblem();
    LastTime = GetTime();
    if (Optimum == MINUS_INFINITY)
        Optimum = PLUS_INFINITY;
    MSTSP_transform();
    LowerBound = TwoMax();
    if ((Bound = BBSSP(LowerBound)) > LowerBound)
        LowerBound = Bound;
    if (ProblemType == ATSP) {
        if ((Bound = BBSSPA(LowerBound)) > LowerBound)
            LowerBound = Bound;
        if ((Bound = BSCSSP(LowerBound)) > LowerBound)
            LowerBound = Bound;
        if ((Bound = BAP(LowerBound)) > LowerBound)
            LowerBound = Bound;
    }
    Time = fabs(GetTime() - LastTime);
    printff("Upper bound = %d, Time = %0.2fs\n", M - LowerBound, Time);
    assert(BestTour = (int *) malloc((DimensionSaved + 1) * sizeof(int)));
    if (Optimum != PLUS_INFINITY)
        Optimum = M - Optimum;
    Bottleneck = M - SolveBTSP(LowerBound);
    if (Optimum != PLUS_INFINITY)
        Optimum = M - Optimum;
    TraceLevel = 1;
    if (OutputTourFileName)
        WriteTour(OutputTourFileName, BestTour, BestCost);
    if (TourFileName && (Optimum == PLUS_INFINITY || Bottleneck > Optimum))
        WriteTour(TourFileName, BestTour, BestCost);
    printff("Value = %d", Bottleneck);
    if (Optimum != MINUS_INFINITY && Optimum != 0)
        printff(", Error = %0.2f%%",
                100.0 * (Optimum - Bottleneck) / Optimum);
    printff(", Time = %0.2f sec.\n\n",
            GetTimeUsage(RUSAGE_SELF) + GetTimeUsage(RUSAGE_CHILDREN));
    return 0;
}

static void MSTSP_transform()
{
    int d, i, j;
    Node *From;

    M = INT_MIN;
    for (i = 1; i <= DimensionSaved; i++) {
        From = &NodeSet[i];
        for (j = ProblemType != ATSP ? i + 1 : 1; j <= DimensionSaved; j++) {
            if (i == j)
                continue;
            d = ProblemType != ATSP ? Distance(From,
                                               &NodeSet[j]) : From->C[j];
            if (d > M)
                M = d;
        }
    }
    if (ProblemType != ATSP) {
        OriginalDistance = Distance;
        Distance = Distance_SPECIAL;
        c = 0;
    } else {
        for (i = 1; i <= DimensionSaved; i++) {
            From = &NodeSet[i];
            for (j = 1; j <= DimensionSaved; j++)
                if (i != j)
                    From->C[j] = M - From->C[j];
        }
    }
}
