#include "LKH.h"
#include <unistd.h>

GainType SolveTSP(int Dimension, char *ParFileName,
                  char *TourFileName, int *Tour);

int SolveTransformedTSP(int Low, int High, int *Tour, GainType *Cost)
{
    FILE *ParFile, *stdoutSaved;
    char ParFileName[256], NewTourFileName[256], Prefix[256];
    Node *From, *To;
    int Bottleneck, d, i;

    sprintf(Prefix, "%s.pid%d", Name, getpid());

    /* Create the parameter file */
    sprintf(ParFileName, "TMP/%s.par", Prefix);
    stdoutSaved = stdout;
    assert(stdout = ParFile = fopen(ParFileName, "w"));
    PrintParameters();
    stdout = stdoutSaved;
    fprintf(ParFile, "LOW = %d\n", Low);
    fprintf(ParFile, "HIGH = %d\n", High);
    fprintf(ParFile, "OPTIMUM = 0\n");
    sprintf(NewTourFileName, "TMP/%s.temp.tour", Prefix);
    fprintf(ParFile, "TOUR_FILE = %s\n", NewTourFileName);
    if (TraceLevel == 0)
        fprintf(ParFile, "TRACE_LEVEL = 1\n");
    if (High == INT_MAX) {
        fprintf(ParFile, "GAIN23 = NO\n");
        fprintf(ParFile, "MAX_CANDIDATES = 0\n");
        fprintf(ParFile, "MOVE_TYPE = 3\n");
        fprintf(ParFile, "PATCHING_A = 1\n");
        fprintf(ParFile, "PATCHING_C = 0\n");
        fprintf(ParFile, "PRECISION = 1\n");
        fprintf(ParFile, "RESTRICTED_SEARCH = NO\n");
        fprintf(ParFile, "SUBGRADIENT = NO\n");
        fprintf(ParFile, "SUBSEQUENT_MOVE_TYPE = %d\n",
                ProblemType == ATSP ? 3 : 2);
    } else if (CoordType == TWOD_COORDS)
        fprintf(ParFile, "CANDIDATE_SET_TYPE = DELAUNAY\n");
    fclose(ParFile);

    /* Solve the TSP/ATSP */
    SolveTSP(DimensionSaved, ParFileName, NewTourFileName, Tour);
    unlink(ParFileName);

    /* Compute the bottleneck cost of the tour */
    for (i = 1; i <= DimensionSaved; i++)
        NodeSet[Tour[i - 1]].Suc = &NodeSet[Tour[i]];
    Bottleneck = INT_MIN;
    *Cost = 0;
    From = FirstNode;
    do {
        To = From->Suc;
        d = ProblemType != ATSP ? Distance(From, To) : From->C[To->Id];
        if (d > Bottleneck)
            Bottleneck = d;
        *Cost += d;
    } while ((From = To) != FirstNode);
    return Bottleneck;
}
